/**
 * M1k2 fader toolkit
 * @copyright Florian Faber, florian.faber@directout.eu
 * @version 1 (02.09.2011) 
 **/

var fader_0 = 129;
var fader_infty = 0;

var db_to_px = [-64.44, -60.96, -57.89, -55.14, -52.66, -50.39, -48.31, -46.38, -44.58, -42.90, -41.32, -39.83, -38.42, -37.08, -35.81, -34.60, -33.44, -32.33, -31.27, -30.25, -29.26, -28.32, -27.40, -26.52, -25.66, -24.84, -24.03, -23.26, -22.50, -21.77, -21.05, -20.36, -19.68, -19.02, -18.38, -17.75, -17.14, -16.54, -15.95, -15.38, -14.82, -14.27, -13.73, -13.21, -12.69, -12.18, -11.69, -11.20, -10.72, -10.25, -9.79, -9.34, -8.89, -8.46, -8.02, -7.60, -7.18, -6.77, -6.37, -5.97, -5.58, -5.19, -4.81, -4.44, -4.07, -3.70, -3.35, -2.99, -2.64, -2.30, -1.96, -1.62, -1.29, -0.96, -0.64, -0.32];


function fader_db_to_gain(dbval) {
	if (Math.round(dbval*10)==0)
		dbval = 0.0;
  var gval = Math.round(2048*Math.pow(10, dbval/20));
	if (gval>65535)
		gval = 65535;
	return gval;
}

function fader_px_to_db(pxval) {
	if (pxval==db_to_px.length) {
		return 0;
	} else if (pxval<db_to_px.length) {
		return db_to_px[pxval];
	} else {
		return -db_to_px[db_to_px.length-(pxval-db_to_px.length)];
	}
}

function fader_db_to_px(dbval) {
	var min = -db_to_px[0];
	var minval = db_to_px.length;
	var v;

	if (dbval>0) {
		for (var i=db_to_px.length; i>=0; i--) {
			v = Math.abs(dbval+db_to_px[i]);
			if (v>=min) {
				i = 0;
			} else {
				min = v;
				minval = db_to_px.length+(db_to_px.length-i);
			}
		}

	} else if (dbval<0) {
		if (dbval<db_to_px[0])
			return 0;

		for (var i=0; i<db_to_px.length; i++) {
			v = Math.abs(dbval-db_to_px[i]);
			if (v>=min) {
				i = db_to_px.length;
			} else {
				min = v;
				minval = i;
			}
		}
	} 

	return minval;
}


function fader_gain_to_db(gainval) {
  return 20*(Math.log(gainval/2048.0)/2.30258509299);
}


function fader_set_knob(fader, db) {
	if (db<-70)
		fader.children[0].style.top = '150px';
	else	
		fader.children[0].style.top = (fader_0-fader_db_to_px(db)+20)+'px';
}


function fader_set_gain(fader, px) {
	var db;

	if (!fader)
		return;

	if (px>=130) {
		px = 130;
		db = -200;
	} else {
		db = fader_px_to_db(fader_0-px);
		if (db>30)
			db = 30;
	}

	if (isNaN(db))
		return; // better safe than sorry

	if (!fader.knob)
		fader = fader.parentElement;
	
	fader_set_knob(fader, db);
	fader.callback(fader.channel, db);
}


function fader_drag_active(fader) {
	return fader.knob_drag;
}


//
// events
//

function fader_mouse_down(e) {
  var x, y;
  var srcElement;

  if (e.srcElement) {
    srcElement = e.srcElement;
  } else if (e.originalTarget) {
    srcElement = e.originalTarget;
  }
	
	if (typeof(e.offsetX)=='number') {
		x = e.offsetX;
		y = e.offsetY;
	} else {
		x = e.layerX;
		y = e.layerY;
	}

  if (e.which==1) {
		if (e.ctrlKey) {
			fader_set_gain(srcElement, 53);
			e.preventDefault();
			return;
		}

		if (srcElement.knob) {
			// click on fader backdrop
			srcElement.knob_drag = true;
			fader_set_gain(srcElement, y-40);
			srcElement.down_y = e.pageY-srcElement.knob.offsetTop;
			srcElement.off_y = 20;

		} else {
			// click on fader knob
			srcElement.parentElement.knob_drag = true;
			srcElement.parentElement.down_y = e.pageY-srcElement.offsetTop;
			srcElement.parentElement.off_y = y;
		}

  } else if (e.which==2) {
    fader_set_gain(srcElement, 53);
		e.preventDefault();
	}
}

function fader_mouse_up(e) {
  var srcElement;

  if (e.srcElement) {
    srcElement = e.srcElement;
  } else if (e.originalTarget) {
    srcElement = e.originalTarget;
  }

	if (srcElement.knob) {
		// click on fader backdrop
		srcElement.knob_drag = false;

	} else {
		// click on fader knob
		srcElement.parentElement.knob_drag = false;
	}
}


function fader_mouse_move(e) {
  var x, y;
  var srcElement;

  if (e.srcElement) {
    srcElement = e.srcElement;
  } else if (e.originalTarget) {
    srcElement = e.originalTarget;
  }
	
	if (srcElement.knob_drag || srcElement.parentElement.knob_drag) {
		if (typeof(e.offsetX)=='number') {
			x = e.offsetX;
			y = e.offsetY;
		} else {
			x = e.layerX;
			y = e.layerY;
		}
	
		
    if (srcElement.knob) {
			if (e.which==0) {
				srcElement.knob_drag = false;

			} else {
				fader_set_gain(srcElement, y-srcElement.off_y-20);
			}

    } else {
			if (e.which==0) {
				srcElement.parentElement.knob_drag = false;
			
			} else {
				fader_set_gain(srcElement.parentElement, e.pageY-srcElement.parentElement.down_y-20);
			}
    }
  }
}


function fader_mouse_wheel(e) {
	var delta = 0;
  var srcElement;

  if (e.srcElement) {
    srcElement = e.srcElement;
  } else if (e.originalTarget) {
    srcElement = e.originalTarget;
  }
	
  if (e.wheelDelta) {
		if (e.wheelDelta>0) {
			delta = -1;
		} else {
			delta = 1;
		}
	} else if (e.detail) {
    if (e.detail==3) {
			delta = 1;
		} else if (e.detail==-3) {
			delta = -1;
		}
	}

  if (srcElement.knob) {
    fader_set_gain(srcElement, srcElement.knob.offsetTop-20+delta);
    e.preventDefault();
  } else if (e.eventPhase==2) {
    fader_set_gain(srcElement.parentElement, srcElement.offsetTop-20+delta);
    e.preventDefault();
  }
}


function fader_mono_popup(container, id, ch, gain, x, y, label, callback) {
	var div = document.getElementById(id);
	if (div) {
		div.style.display = 'block';

	} else {
		div = document.createElement('div');
		div.style.position = 'absolute';
		div.style.width = '22px';
		div.style.height = '196px';
		div.style.backgroundImage = 'url(/img/fader_mono_popup.png)';
		div.style.left = (x+1)+'px';
		div.style.top = (y-196)+'px';
		div.style.zIndex = 6;
		div.id = id;
		div.addEventListener('mousemove', fader_mouse_move, true);
		div.addEventListener("DOMMouseScroll", fader_mouse_wheel, true);
		div.addEventListener('mousewheel', fader_mouse_wheel, false);
		div.onmousedown = fader_mouse_down;
		div.onmouseup = fader_mouse_up;
		div.channel = ch;
		div.callback = callback;
		
		fader_knob = document.createElement('div');
		fader_knob.style.position = 'absolute';
		fader_knob.style.width = '20px';
		fader_knob.style.height = '40px';
		fader_knob.style.backgroundImage = 'url(/img/fader_knob.png)';
		fader_knob.style.left = '1px';
		fader_knob.style.zIndex = 7;
		fader_knob.style.cursor = 'pointer';
		fader_knob.addEventListener("DOMMouseScroll", fader_mouse_wheel, true);
		fader_knob.addEventListener('mousewheel', fader_mouse_wheel, false);
		div.knob = fader_knob;
		div.appendChild(fader_knob);
		
		
		var lbl = document.createElement('div');
		lbl.style.position = 'absolute';
		lbl.style.width = '22px';
		lbl.style.height = '13px';
		lbl.style.background = '#cc0000';
		lbl.style.left = '0px';
		lbl.style.top = '1px';
		lbl.style.fontSize = '10px';
		lbl.style.textAlign = 'center';
		lbl.style.zIndex = 7;
		lbl.innerHTML = label;
		div.appendChild(lbl);

		container.appendChild(div);
	}
	
  fader_set_knob(div, gain);
}
