var gain_canvas, gain_ctx, sel_canvas, sel_ctx, perm_canvas, perm_ctx;
var channel_info, display_labels;

var last_sel = -1;
var sel_cnt = 0;

var color_grid = "rgb(0, 0, 0)";
var color_listen = "rgba(255, 255, 255, 0.6)";
var color_sel = "rgba(255, 255, 255, 0.5)";
var color_blink = "rgba(255, 255, 255, 0.7)";
var color_blink_err = "rgba(255, 0, 0, 0.7)";
var color_dead = "rgba(66, 66, 66, 0.4)";
var color_disabled = "rgb(90, 90, 90)";

var color_gain_white = "rgb(255, 255, 255)";
var color_gain_black = "rgb(0, 0, 0)";
var color_gain_red = "rgb(255, 0, 0)";
var color_gain_pos = "rgba(225, 215, 62, 0.7)";
var color_gain_neg = "rgba(62, 166, 31, 0.7)";
var color_gain_uni = "rgba(100, 100, 100, 0.7)";
var color_gain_locked = "rgba(100, 50, 50, 0.5)";

var img_locked = false;


function draw_gain(ch, db) {
	var port = Math.floor(ch/64);
	var pch = ch%64;
	var mul = parent.frames[0].get_effective_mul(port)+1;
	if (mul==3) 
		mul=4;

	if (pch*mul>=64)
		return;

	if ((parent.frames[0].status_port_mode[port]==56) && 
			(pch*mul >= 56)) 
		return;

	var x = pch*mul*20;
	var y = port*40;
	
	if (pch*mul>=32) {
		x -= 640;
		y += 20;
	}

	gain_ctx.font = 'bold 9px sans-serif';
	gain_ctx.textBaseline = 'top';

	var str = "";
	if (db<-60.21) {
		str = '\u221E';
	} else {
		var dbval = Math.round(Math.abs(db*10));
		str = dbval/10;
	}

	gain_ctx.clearRect(x+1, y+1, 20*mul-1, 19);
	
	m = gain_ctx.measureText(str);
	offset = (20*mul-m.width)/2;

	if (db<0) {
		gain_ctx.fillStyle = color_gain_neg;
		gain_ctx.fillRect(x+1, y+1, 20*mul-1, 19);
		
		gain_ctx.fillStyle = color_gain_white;
		gain_ctx.fillText(str, offset+x+1, y+6);
		
	} else if (db>0) {
		gain_ctx.fillStyle = color_gain_pos;
		gain_ctx.fillRect(x+1, y+1, 20*mul-1, 19);
		
		gain_ctx.fillStyle = color_gain_red;
		gain_ctx.fillText(str, offset+x+1, y+6);
	}

	if (parent.frames[0].config_gain_lock[ch]!=0) {
		/*
			gain_ctx.fillStyle = color_gain_locked;
			gain_ctx.fillRect(x+1, y+1, 20*mul-1, 19);		
		*/

		gain_ctx.drawImage(img_locked, x+1, y+1);
	}
}


function draw_portgain(port, db) {
	var o = document.getElementById('portgain_'+port);

	if (db>0) {
		o.style.backgroundColor = '#e1d73f';
		o.style.color = '#ff0000';
	} else if (db<0) {
		o.style.backgroundColor = '#3ea61f';
		o.style.color = '#ffffff';
	} else {
		o.style.backgroundColor = '';
		o.style.color = '#000000';
	}

	if (db<-67) {
		o.innerHTML = '&#8734;';
	} else if (Math.abs(db)>10) {
		o.innerHTML = Math.round(Math.abs(db));
	} else {
		o.innerHTML = Math.round(Math.abs(db*10))/10;
	}
}


function redraw_port(port) {
	var bg = document.getElementById("bg_"+port);
	if (bg)
		bg.src = "/img/moni_bg_"+parent.frames[0].get_effective_mul(port)+"_0.png";	
}


function draw_grid(port) {	
	var mul = parent.frames[0].get_effective_mul(port)+1;
	if (mul==3) 
		mul=4;
	
	gain_ctx.fillStyle = color_grid;
	gain_ctx.fillRect(0, port*40, 640, 1);
	gain_ctx.fillRect(0, port*40+20, 640, 1);
	gain_ctx.fillRect(0, port*40+40, 640, 1);

	for (var x=0; x<=32/mul; x++)
		gain_ctx.fillRect(x*mul*20, port*40, 1, 41);

	if (parent.frames[0].status_port_mode[port]==56) {
		gain_ctx.fillStyle = color_dead;
		gain_ctx.fillRect(481, port*40+20+1, 160, 19);
	}
	
}


function update_channel_info(port, ch) {
	var mul = parent.frames[0].get_effective_mul(port)+1;
	if (mul==3) 
		mul=4;

	if ((parent.frames[0].status_port_mode[port]==56) &&
			(ch*mul>=56)) {
		channel_info.style.display = 'none';
		return;
	}

	var x = ch*mul*20;
	var y = port*40+60;
	
	if (ch*mul>=32) {
		x -= 640;
		y += 20;
	}

	var v = parent.frames[0].config_gains[port*64+ch];
	var str = "0";
	var db = fader_gain_to_db(parent.frames[0].config_gains[port*64+ch]);
	if (db<-60.21) {
	  str = '&#8734;';
	} else if (db>=-60.21) {
	  var dbval = Math.round(Math.abs(db*10));
	  str = ((db>0)?"+":(db<0)?"-":"")+(dbval/10);
	}

	if (display_labels) {
		channel_info.innerHTML = "<span style='background-color: #"+parent.frames[0].labels_output_ports_bg[port]+"; color: #"+parent.frames[0].labels_output_ports_fg[port]+";'>"+parent.frames[0].labels_output_ports[port]+"</span> / <span style='background-color: #"+parent.frames[0].labels_audio_out_bg[ch]+"; color: #"+parent.frames[0].labels_audio_out_fg[ch]+";'>"+parent.frames[0].labels_audio_in[ch]+"</span> ("+str+" dB)";

	} else {
		channel_info.innerHTML = "Port "+(port+1)+", Ch "+(ch+1)+": "+str+" dB";
	}

	channel_info.style.display = 'block';

	if (x<100)
		x = 100;

	if (y+channel_info.clientHeight>640)
		y -= 100+channel_info.clientHeight;

	channel_info.style.top = y+'px';
	channel_info.style.left = x+'px';
}


function draw_sel(ch) {
	var port = Math.floor(ch/64);
	ch = ch%64;
	var mul = parent.frames[0].get_effective_mul(port)+1;
	if (mul==3) 
		mul=4;

	if (ch*mul>=64)
		return;

	/*
	if ((parent.frames[0].status_port_mode[port]==56) && 
			(ch*mul >= 56)) 
		return;
	*/

	var x = ch*mul*20;
	var y = port*40;
	
	if (ch*mul>=32) {
		x -= 640;
		y += 20;
	}

	//sel_ctx.strokeRect(x+1, y+1, 20*mul-2, 19);
	sel_ctx.fillRect(x, y, 20*mul+1, 21);
}


function selcnt_redraw() {
	sel_cnt = 0;
	portsel_cnt = 0;

	for (var i=0; i<1024; i++) {
	  if (parent.frames[0].gain_sel[i] > 0) {
	    draw_sel(i);
	    sel_cnt++;
	  }
	}

	for (var i=0; i<16; i++) {
	  if (parent.frames[0].portgain_sel[i]>0)
	    portsel_cnt++;
	}

	var o = document.getElementById("selcnt");
	if (o) {
	  var str = "";
	  if (portsel_cnt==0) {
	    str = "No ports, ";
	  } else if (portsel_cnt==1) {
	    str = "1 port, ";
	  } else {
	    str = portsel_cnt+" ports, ";
	  }
	  if (sel_cnt==0) {
	    str += "no channels selected.";
	  } else if (sel_cnt==1) {
	    str += "1 channel selected.";
	  } else {
	    str += sel_cnt+" channels selected.";
	  }
	  o.innerHTML = str;
	}

	var sel = ((portsel_cnt>0) || (sel_cnt>0));
	document.getElementById("btn_plus").disabled = !sel;
	document.getElementById("btn_minus").disabled = !sel;
	document.getElementById("btn_set").disabled = !sel;
	document.getElementById("btn_lock").style.opacity = sel?"1.0":"0.5";
	document.getElementById("btn_unlock").style.opacity = sel?"1.0":"0.5";

}

function selmap_redraw() {
	sel_ctx.clearRect(0, 0, 641, 641);
	sel_ctx.strokeStyle = color_sel;
	sel_ctx.fillStyle = color_sel;
	sel_ctx.lineWidth = 1;
	selcnt_redraw();
}


function selmap_clear() {
  for (var i=0; i<1024; i++)
    parent.frames[0].gain_sel[i] = 0;
}


function selportgain_clear() {
  for (var i=0; i<16; i++)
    parent.frames[0].portgain_sel[i] = 0;
}

function selportgain_redraw() {
  for (var i=0; i<16; i++) {
    var o = document.getElementById('portgain_'+i);
    if (parent.frames[0].portgain_sel[i]==1) 
      o.style.border = '1px solid red';
    else
      o.style.border = '1px solid grey';
  }
  selcnt_redraw();
}


//
// event callbacks
//


var last_port, last_ch;


function canvas_mousemove(e) {
	var port = Math.floor(e.layerY/40);

	if (port>15)
		return; 

	var mul = parent.frames[0].get_effective_mul(port)+1;
	if (mul==3) 	
	mul=4;
	
	if (e.layerX>639)
		return;

	var ch = Math.floor((e.layerX)/20/mul);
	
	if (e.layerY%40>20)
		ch += 32/mul;

	if ((port != last_port) || (ch != last_ch)) {
		//draw_selection(last_port, last_ch, false);

		if (parent.frames[0].permissions.gain[port*64+ch]) {
			sel_canvas.style.cursor = 'pointer';
		} else {			
			sel_canvas.style.cursor = 'not-allowed';
		}
		update_channel_info(port, ch);

		last_port = port;
		last_ch = ch;
	}
}


function ch_select(ch) {
  if (parent.frames[0].permissions.gain[ch]==0)
    return;

  var port = Math.floor(ch/64);
  ch = ch%64;
  
  var mul = parent.frames[0].get_effective_mul(port)+1;
  if (mul==3) 	
    mul=4;
  
  if (ch*mul>=64)
    return;
  
  parent.frames[0].gain_sel[port*64+ch] = 1;
}


function port_mousedown(e) {
  if (e.which!=1)
    return;

  if (!e.ctrlKey) {
    if (last_sel > -1) {
      selmap_clear();
      selmap_redraw();
    }
  }

  var srcElement;

  if (e.srcElement) {
    srcElement = e.srcElement;
  } else if (e.originalTarget) {
    srcElement = e.originalTarget;
  }

	if (!e.ctrlKey && parent.frames[0].portgain_sel[srcElement.portid]==0)
		selportgain_clear();

	if (parent.frames[0].permissions.portgain[srcElement.portid]) {
		parent.frames[0].portgain_sel[srcElement.portid] = (parent.frames[0].portgain_sel[srcElement.portid]=="1")?0:1;

		selportgain_redraw();
	}
}


function canvas_mousedown(e) {
	switch (e.which) {
	case 1:
		var cur = last_port*64 + last_ch;
		if (!parent.frames[0].permissions.gain[cur])
			return;

		if (e.ctrlKey) {
			if (parent.frames[0].gain_sel[last_port*64+last_ch]>0) {
				parent.frames[0].gain_sel[last_port*64+last_ch] = 0;
			} else {
				parent.frames[0].gain_sel[last_port*64+last_ch] = 1;
				last_sel = last_port*64+last_ch;
			}

		} else if (e.shiftKey) {
		  selportgain_clear();
			if (last_sel > -1) {
				selmap_clear();

				if (cur<last_sel) {
					for (var i=cur; i<=last_sel; i++)
						ch_select(i);
				} else {
					for (var i=last_sel; i<=cur; i++)
						ch_select(i);
				}

			} else {
				if (parent.frames[0].gain_sel[last_port*64+last_ch]>0) {
					parent.frames[0].gain_sel[last_port*64+last_ch] = 0;
				} else {
					parent.frames[0].gain_sel[last_port*64+last_ch] = 1;
					last_sel = last_port*64+last_ch;
				}
			}

		} else {
		  selportgain_clear();
			if (cur != last_sel) {
				selmap_clear();
				parent.frames[0].gain_sel[last_port*64+last_ch] = 1;
				last_sel = cur;

			} else {
				if (parent.frames[0].gain_sel[last_port*64+last_ch]>0) {
					parent.frames[0].gain_sel[last_port*64+last_ch] = 0;
				} else {
					parent.frames[0].gain_sel[last_port*64+last_ch] = 1;
					last_sel = last_port*64+last_ch;
				}
			}
		}

		selmap_redraw();
		selportgain_redraw();
			
		break;

	case 2:
		break;

	case 3:
		break;

	}
}

function canvas_mouseup(e) {

}


function canvas_mouseout(e) {
	channel_info.style.display = 'none';
	/*
	draw_selection(last_port, last_ch, false);
	if ((last_port*64+last_ch)==listen_channel) {
		draw_selection(last_port, last_ch, color_listen);
	}
	*/
	last_port = -1;
	last_ch = -1;
}


function gain_keypress(e) {
	var o = document.getElementById('gainval');

	if (e.target == o)
		return;

	if (((e.charCode>=48) && (e.charCode<=57)) || (e.charCode==46)) {
		o.value += String.fromCharCode(e.charCode);

		e.preventDefault();

	} else if (e.charCode==45) {
		if (o.value.length==0)
			o.value = '-';
		else {
			gain_dec(o.value);
			o.value = '';
		}

		e.preventDefault();

	} else if (e.charCode==43) {
		if (o.value.length==0)
			o.value = '+';
		else {
			gain_inc(o.value);
			o.value = '';
		}

		e.preventDefault();

	} else if (e.charCode==13) {
		if (o.value.length>0)
			gain_set(o.value);
		o.value = '';

		e.preventDefault();

  } else if (e.keyCode==100) { // 'd'
    set_display_labels(!display_labels);
		update_channel_info(last_port, last_ch);
		
		e.preventDefault();
	}
	return false;
}


function portgain_mousedown(e) {
  if (e.which!=1)
    return;

  var srcElement;

  if (e.srcElement) {
    srcElement = e.srcElement;
  } else if (e.originalTarget) {
    srcElement = e.originalTarget;
  }

	selmap_clear();
	
	
}



//
//
//


function get_selected_channels() {
    var args = Array(), argm = 0; 
  for (var i=0; i<1024; i++) {
    if (parent.frames[0].gain_sel[i]==1) {
      if (argm==0) {
	argm = 1;
	args.push(i);
      }
    } else if (argm==1) {
      var start = args[args.length-1];
      if (start<i-1)
	args[args.length-1] = args[args.length-1]+"-"+(i-1);
      argm = 0;
    }
  }
  if (argm==1) {
    var start = args[args.length-1];
    if (start<i-1)
      args[args.length-1] = args[args.length-1]+"-"+(i-1);
  }
  return args;
}


function gain_inc(val) {
  var v = parseFloat(val);
  if (isNaN(v))
    return;
  
  var args = get_selected_channels();
  if (args.length>0) {
    send_cmd("inc_gain?"+v+","+args.toString());
  }
  
  for (var i=0; i<16; i++) {
    if (parent.frames[0].portgain_sel[i]==1) {
      send_cmd("portgain?"+i+","+fader_db_to_gain(fader_gain_to_db(parent.frames[0].config_portgains[i])+v));
    }
  }
  
  val_set = 1;
}


function gain_dec(val) {
  var v = parseFloat(val);
  if (isNaN(v))
    return;
  
  var args = get_selected_channels();
  if (args.length>0) {
    send_cmd("dec_gain?"+v+","+args.toString());
  }
  
  for (var i=0; i<16; i++) {
    if (parent.frames[0].portgain_sel[i]==1) {
      send_cmd("portgain?"+i+","+fader_db_to_gain(fader_gain_to_db(parent.frames[0].config_portgains[i])-v));
    }
  }
  
  val_set = 1;
}

function gain_set(val) {
  var v = parseFloat(val);
  if (isNaN(v))
    return;

  var args = get_selected_channels();
  if (args.length>0) {
    send_cmd("set_gain?"+v+","+args.toString());
  }

  for (var i=0; i<16; i++) {
    if (parent.frames[0].portgain_sel[i]==1) {
      send_cmd("portgain?"+i+","+fader_db_to_gain(v));
    }
  }
  
  val_set = 1;
}

function gain_lock(v) {
	for (var i=0; i<1024; i++) {
		if (parent.frames[0].gain_sel[i]==1) {
			send_cmd("lock_gain?"+i+","+v);
		}
	}
}


//
// eventsource callbacks
//


function ess_port_mode(port, mode) {
	gain_ctx.clearRect(480, port*40+20+1, 160, 19);
	draw_grid(port);
}

function esc_wck_mul(port, mul) {
  if (port<16) {
    gain_ctx.clearRect(0, port*40, 640, 40);
    sel_ctx.clearRect(0, port*40, 640, 40);
    
    draw_grid(port);
    
    redraw_port(port);
  }
}

function esc_gain(ch, gain) {
	draw_gain(ch, fader_gain_to_db(gain));
	if (last_port>=0)
		update_channel_info(last_port, last_ch);
}

function esc_gl(ch, val) {
	draw_gain(ch, fader_gain_to_db(parent.frames[0].config_gains[ch]));
}

function esc_portgain(port, gain) {
	draw_portgain(port, fader_gain_to_db(gain));
}


//
// wwwdata callbacks
//


function cb_display_labels(data) {
	display_labels = (data=='true');
	update_channel_info(last_port, last_ch);
	//document.getElementById('cb_show_labels').checked = display_labels;
}


//
// site callbacks
//


function set_display_labels(val) {
	display_labels = val;
	post_call('/wwwdata?display_labels', val, false);
}

//


function ch_not_permitted(ch) {
  var port = Math.floor(ch/64);
  ch = ch%64;

  var mul = parent.frames[0].get_effective_mul(port)+1;
  if (mul==3) 
    mul=4;

  if ((parent.frames[0].status_port_mode[port]==56) && 
      (ch >= 56)) 
    return;

  if (ch%mul!=0)
    return;

  ch = ch/mul;

  var y = port*40;
  if (ch>=(32/mul))
    y += 20;
  var x = 20*(ch%(64/(mul*2)));
	
  perm_ctx.fillRect(x*mul+1, y+1, 20*mul-1, 19);
}


function permissions_updated() {
  perm_ctx.clearRect(0, 0, 640, 640);
  perm_ctx.fillStyle = color_disabled;

  for (var i=0; i<1024; i++) {
    if (parent.frames[0].permissions.gain[i]==0)
      ch_not_permitted(i);
  }

  for (var i=0; i<16; i++) {
    if (parent.frames[0].permissions.portgain[i]==0) {
      var o = document.getElementById("portgain_"+i);
      o.className = "portgain disabled";
    }
  }
}


//
//
//


function gains_init() {
  var o;
  var labels = document.getElementById("gains_labels");
  var bg = document.getElementById("gains_bg");
  gain_canvas = document.getElementById('gain_canvas');
  gain_ctx = gain_canvas.getContext('2d');
  sel_canvas = document.getElementById('sel_canvas');
  sel_ctx = sel_canvas.getContext('2d');
  perm_canvas = document.getElementById('perm_canvas');
  perm_ctx = perm_canvas.getContext('2d');
  
  channel_info = document.getElementById("channel_info");
  
  sel_canvas.onmousemove = canvas_mousemove;
  sel_canvas.onmousedown = canvas_mousedown;
  sel_canvas.onmouseup = canvas_mouseup;
  sel_canvas.onmouseout = canvas_mouseout;
  
  document.addEventListener('keypress', gain_keypress, false);
  
  var gv = document.getElementById('gainval');
  gv.addEventListener('keypress', gain_keypress, false);

  ajax_call('/wwwdata?display_labels', cb_display_labels);
  
  display_labels = false;
  
  for (var i=0; i<16; i++) {
    var o = document.createElement("img");
    o.id = "bg_"+i;
    o.style.top = (i*40)+"px";
    bg.appendChild(o);
    esc_wck_mul(i, parent.frames[0].get_effective_mul(i));
    ess_port_mode(i, parent.frames[0].status_port_mode[i]);
    
    o = document.createElement("div");
    o.id = "label_"+i;
    o.innerHTML = "Port "+(i+1)+"<div class='portgain' id='portgain_"+i+"'></div>";
    o.style.top = (i*40)+"px";
    labels.appendChild(o);
    
    o = document.getElementById("portgain_"+i);
    o.onmousedown = port_mousedown;
    o.portid = i;
  }
  
  img_locked = new Image();
  img_locked.onload = gains_init2;
  img_locked.src = '/img/gain_locked.png';
}


function gains_init2() {
  for (i=0; i<1024; i++) {
    esc_gain(i, parent.frames[0].config_gains[i]);
  }
  
  for (i=0; i<16; i++) {
    esc_portgain(i, parent.frames[0].config_portgains[i]);
  }
  
  selmap_redraw();
  selportgain_redraw();

  permissions_updated();
}
