/**
 * M1k2 MIDI matrix
 * @copyright Florian Faber, florian.faber@directout.eu
 * @version 2 (05.05.2013) 
 *          - added permissions
 * @version 1 (18.11.2010) 
 **/

var freq_min = 32000;
var bg_low = 'cccc00';
var bg_act = '009000';
var bg_forbidden = '900000';
var bg_nil = '';
var labels;
var input_active = false;


function cb_keydown(e) {
  if (input_active)
    return;
  
  if (e.keyCode==68) { // 'd'
    push_cycle_button(document.getElementById("show_labels"));
    e.preventDefault();

	} else if (e.which==82) { // 'r'
		restore();

	} else if (e.which==83) { // 's'
		snapshot();
  }
}


//
// callback functions for status.js
//


function esc_midixp(p_in, p_out) {
  mini_xp(p_out, p_in);
}


function ess_sync(port, status) {
	ess_freq(port, 0);
}


function ess_freq(port, freq) {
  mini_update_in(port);
  mini_update_out(port);

  bg = (xp_valid(parent.frames[0].config_midi[port], port))?bg_act:bg_low;
  mini_update_xp(port, bg);
}


function esl(type, id, label, fg, bg) {
  if (!labels)
    return;

  if (type==132) { // input port
    mini_update_in(id);

  } else if (type==133) { // output port
    mini_update_out(id);

  }
}


//
// callback functions for minimatrix.js
//


function mini_cb_input(active) {
  input_active = active;
}


function mini_cb_color(dst, src) {
  if (!(mini_cb_enabled('src', src) && mini_cb_enabled('dst', dst)))
    return bg_forbidden;

  if (dst>=16) {
    if (src>=16) {
      return bg_act;
    } else {
      return (parent.frames[0].status_port_freq[src]>=freq_min)?bg_act:bg_low;
    }
  } else if (src>=16) {
    return bg_act;
  } else {
    return (xp_valid(src, dst))?bg_act:bg_low;
  }
}

function mini_cb_cursor(dst, src) {
  if (mini_cb_enabled('src', src) && mini_cb_enabled('dst', dst))
    return "pointer";

  return "not-allowed";
}


function mini_cb_click(dst, src) {
  send_cmd("midixp?"+dst+","+src);
}


function mini_cb_label_editable(type, id) {
  return (labels==1);
}


function mini_cb_label_changed(type, id, newlabel, fg, bg) {
  post_cmd("label?"+((type=='src')?132:133)+","+id+","+fg+","+bg, newlabel);
}


var	names_hw = ["Port 1", "Port 2", "Port 3", "Port 4",
		    "Port 5", "Port 6", "Port 7", "Port 8",
		    "Port 9", "Port 10", "Port 11", "Port 12",
		    "Port 13", "Port 14", "Port 15", "Port 16",
		    "Front MIDI", "Controller"];


function mini_cb_label(type, id) {
  if (id==-1) {
    return "N/C";
  } else if (id>=16) {
    return names_hw[id];
  } else {
    if (labels==1) {
      if (type=='src')
	return parent.frames[0].labels_midi_in[id];
      else
	return parent.frames[0].labels_midi_out[id];
    } else if (labels==2) {
      if (type=='src')
	return parent.frames[0].labels_input_ports[id];
      else
	return parent.frames[0].labels_output_ports[id];
    } else {
      return names_hw[id];
    }
  }
}


function mini_cb_enabled(type, id) {
  if (type=='src') {
    if (id==-1)
      return true;
    return (parent.frames[0].permissions.midi_input[id]==1);
  } else {
    return (parent.frames[0].permissions.midi_output[id]==1);
  }
}


function mini_cb_bg(type, id) {
  if ((type=='src') && (id<16) && (parent.frames[0].status_port_freq[id]<freq_min)) {
		return bg_low;

  } else {  
		if (labels==1) {
			if (type=='src') 
				return parent.frames[0].labels_midi_in_bg[id];
			else
				return parent.frames[0].labels_midi_out_bg[id];
		} else if (labels==2) {
			if (type=='src')
				return parent.frames[0].labels_input_ports_bg[id];
			else
				return parent.frames[0].labels_output_ports_bg[id];
		} else {
			return '';
		}
	}
}


function mini_cb_fg(type, id) {
  if (labels==1) {
    if (type=='src') 
      return parent.frames[0].labels_midi_in_fg[id];
    else
      return parent.frames[0].labels_midi_out_fg[id];
  } else if (labels==2) {
    if (type=='src')
      return parent.frames[0].labels_input_ports_fg[id];
    else
      return parent.frames[0].labels_output_ports_fg[id];
  } else {
    return '';
  }
}


function mini_cb_perm(type, id) {
	if (type=='src')
		return perm_midi_in[id];
	else
		return perm_midi_out[id];
}


var last_activity = Array(18);

function signals(data) {
		if (data.length<131)
				return;
		
		for (var i=0; i<18; i++) {
				var bytepos = Math.floor(i/8);
				var bitpos = i%8;
				var sig = ((data[bytepos+128]&(1<<bitpos))>0)?1:0;
				if (sig!=last_activity[i]) {
						update_activity(i, sig);
						last_activity[i] = sig;
				}
		}		
}


//
// internal
//


function update_activity(port, val) {
		var o = document.getElementById("act_"+port);
		if (o) {
				o.src = (val)?'/img/act_on.png':'/img/act_off.png';
		}
}

function show_labels(v) {
  if (typeof(v)=='string') {
    v = parseInt(v, 10);
    if ((typeof(v)!='number') || isNaN(v))
      v = 0;
    set_cycle_button(document.getElementById("show_labels"), v);
  } else if (labels != v) {
    post_call('/wwwdata?midimatrix_labels', v, false);
  }

  labels = v;
  
  for (i=0; i<16; i++) {
    mini_update_in(i);
    mini_update_out(i);
  }
}


function set_11() {
  for (i=0; i<18; i++)
    send_cmd("midixp?"+i+","+i);
}

function set_nc() {
  for (i=0; i<18; i++) 
    send_cmd("midixp?"+i+",-1");
}


function xp_valid(src, dst) {
  return (parent.frames[0].status_port_freq[src]>=freq_min);
}

function permissions_updated() {
  mini_update_all();
	for (var i=0; i<18; i++)
    esc_midixp(parent.frames[0].config_midi[i], i);
}

function midimatrix_init() {
    if (parent.frames[0].status_board>1) {
	document.getElementById("activity").style.display = "block";
	rt_connect();
    }
    
    ajax_call('/wwwdata?midimatrix_labels', show_labels);
    
    init_minimatrix(18);
    
    
    for (i=0; i<18; i++) 
	esc_midixp(parent.frames[0].config_midi[i], i);
    
    for (i=0; i<16; i++) 
	ess_freq(i, parent.frames[0].status_port_freq[i]);
    
    window.onkeydown = cb_keydown;
    
    init_cycle_button(document.getElementById("show_labels"), ['System Labels', 'User Labels'], show_labels);
    
    permissions_updated();
}

window.onload = midimatrix_init;
