/**
 * Port configuration
 * @copyright D.O.TEC
 * @author Florian Faber, florian.faber@directout.eu
 * @version 3 (30.03.2013)
 **/


//
//  sync log
//

var sync_canvas = null;
var sync_ctx = null;
var sync_table = false;
var sync_uptime = false;
var sync_update_rate = 1;
var sync_width = 400;
var sync_last = -1;
var color_grid = "rgb(0, 0, 0)";
var color_bg = "rgb(220, 220, 220)";
var color_sync = "rgba(0, 255, 0, 0.7)";
var color_lock = "rgba(255, 0, 0, 0.7)";
var color_nosignal = "rgba(0, 0, 0, 0.7)";
var color_event = "rgb(50, 50, 50)";
var oldmodules = false;
var oldlevels = false;
var port = 0;

function sync_log_draw() {
    if (!sync_table)
				return;
    if (!sync_uptime)
				return;

    var now = new Date().getTime()/1000;
    var span;
    if (parent.frames[0].history_scale==0)
				span = now-sync_uptime;
    else
				span = parent.frames[0].history_scale;

    var scale = span/sync_width;
    
    sync_ctx.fillStyle = color_bg;
    sync_ctx.fillRect(0, 0, sync_width, 100);
    
    sync_ctx.lineWidth = 10;

    var off_x, off_y, c;
    var last_x = 0;
    
    sync_ctx.fillStyle = color_event;
    for (var i=0; i<sync_table.length; i++) {
				if (sync_table[i][1]==sync_table[i][2])
						continue;

				off_x = (sync_table[i][0]-(now-span))/scale;
				sync_ctx.fillRect(off_x, 30, 1, 70);

				if (i==0)
						last_x = off_x;
    }

    for (var i=1; i<sync_table.length; i++) {
				off_x = (sync_table[i][0]-(now-span))/scale;

				switch (sync_table[i-1][2]) {
				case 0:
						off_y = 85;
						c = color_nosignal;
						break;
				case 1:
						off_y = 70;
						c = color_lock;
						break;
				case 3:
						off_y = 55;
						c = color_sync;
						break;
				}
				
				sync_ctx.fillStyle = c;
				
				if (off_x-last_x>=9)
						sync_ctx.fillRect(last_x, off_y, off_x-last_x, 9);
				else
						sync_ctx.fillRect(last_x-4, off_y, 9, 9);
				
				last_x = off_x;
    }
    
    if (sync_table.length>=i) {
				switch (sync_table[i-1][2]) {
				case 0:
						off_y = 85;
						c = color_nosignal;
						break;
				case 1:
						off_y = 70;
						c = color_lock;
						break;
				case 3:
						off_y = 55;
						c = color_sync;
						break;
				}
    }

    sync_ctx.fillStyle = c;
    sync_ctx.fillRect(last_x, off_y, 400-last_x, 9);

    var gridscale = 0;
    var gridstr = "";

    sync_ctx.fillStyle = color_grid;
    if (span<=60) {
				gridscale = 10;
				gridstr = '10s';
				sync_update_rate = 1;
    } else if (span<600) {
				gridscale = 60;
				gridstr = '1m';
				sync_update_rate = 1;
    } else if (span<3600) {
				gridscale = 300;
				gridstr = '5m';
				sync_update_rate = 10;
    } else if (span<10800) {
				gridscale = 900;
				gridstr = '15m';
				sync_update_rate = 30;
    } else if (span<86400) {
				gridscale = 3600;
				gridstr = '1h';
				sync_update_rate = 60;
    } else {
				for (var h=2;;h++) {
						if (span<86400*h) {
								gridscale = 3600*h;
								gridstr = h+'h';
								break;
						}
				}
				sync_update_rate = 60;
    }
    
    document.getElementById('scaleinfo').innerHTML = '1 Tick = '+gridstr;
    for (var i=0; i<span; i+=gridscale) {
				sync_ctx.fillRect(400-(i/scale), 0, 1, 12); 
    }
}


function get_timestamp(d) {
    return d.getFullYear()+"-"+((d.getMonth()<9)?'0':'')+(d.getMonth()+1)+"-"+((d.getDate()<10)?'0':'')+d.getDate()+" "+((d.getHours()<10)?'0':'')+d.getHours()+":"+((d.getMinutes()<10)?'0':'')+d.getMinutes()+":"+((d.getSeconds()<10)?'0':'')+d.getSeconds();
}


function sync_table_update() {
    var t = document.getElementById('sync_table');
    while (t.rows.length>1)
				t.deleteRow(1);

    for (var i=sync_table.length-1; i>=0; i--) {
				var row = t.insertRow(t.rows.length);
				row.className = (i%2==0)?"even":"odd";
				var c = row.insertCell(0);
				c.className = 'timestamp';
				c.innerHTML = get_timestamp(new Date(sync_table[i][0]*1000));
				c = row.insertCell(1);
				c.style.textAlign = 'center';
				switch (sync_table[i][2]) {
				case 0:
						c.style.backgroundColor = '#000';
						c.style.color = '#eee';
						c.innerHTML = 'No signal';
						break;
				case 1:
						c.style.backgroundColor = '#f00';
						c.style.color = '#000';
						c.innerHTML = 'Lock';
						break;
				case 3:
						c.style.backgroundColor = '#0f0';
						c.style.color = '#000';
						c.innerHTML = 'Sync';
						break;
				}
    }
}

function sync_log_data(str) {
    sync_table = JSON.parse(str);
    if (sync_table && sync_table.length>0)
				sync_last = sync_table[sync_table.length-1][1];
    sync_log_draw();
    sync_table_update();
}

function sync_log_uptime(str) {
    sync_uptime = (new Date().getTime()/1000) - parseInt(str, 10);
    sync_log_draw();
}

function sync_log_init(port) {
    sync_table = Array();
    sync_table_update();
    sync_last = -1;
    sync_canvas = document.getElementById('sync_log_graph');
    sync_canvas.width = 400;
    sync_canvas.height = 100;
    sync_ctx = sync_canvas.getContext('2d');
    ajax_call('/data/uptime', sync_log_uptime);
    ajax_call('/data/sync_history?'+port, sync_log_data);
}


//
//
//


/**
 * Returns 0 - no signal
 *         1 - signal, but (potentially) asynchronous
 *         3 - sync
 **/
function sync_check(ch_in, ch_out, input) {
    var port_in = Math.floor(ch_in/64);
    var port_out = Math.floor(ch_out/64);
    var sch_in = ch_in-port_in*64;
    var sch_out = ch_out-port_out*64;

    if (port_in==port_out)
				return 3;
    
    if (input) {
				// check if input signal is present and sane
				var sync = parent.frames[0].status_port_sync;
				var s_in = parent.frames[0].status_port_sync[port_in];
				if (parent.frames[0].status_port_sync[port_in]<3)
						return parent.frames[0].status_port_sync[port_in];
    }

    if (parent.frames[0].get_effective_mul(port_in)!=parent.frames[0].get_effective_mul(port_out))
				return 1;

    var eff_source_in = parent.frames[0].get_effective_source(port_in);
    var eff_source_out = parent.frames[0].get_effective_source(port_out);
    var is_sync = 1;
    if (input) {
				if (eff_source_in<16) {
						// check if MADI input that is used as reference is sane
						is_sync = (parent.frames[0].status_port_sync[eff_source_in]==3);
				}
    }
    if ((eff_source_in==eff_source_out) && is_sync)
				return 3; // everything sane & sync
    return 1; // sync sources differ or wordclock reference not sane
}


function get_sync_color(state) {
    switch (state) {
    case 1:
				return '#ff0';
    case 3:
				return '#eee';
    default:
				return '#f00';
    }
}


function update_redundancy_details() {
    var c = document.getElementById("redundancy_details");
    var status = parent.frames[0].get_redundancy_status(port);
    var str = "";

    if (status==0) {
				c.style.color = 'green';
				c.style.fontWeight = 'bold';
				str = "OK";
    } else if (status>0) {
				c.style.color = 'red';
				c.style.fontWeight = 'bold';
				str = parent.frames[0].verbose_redundancy_check(status);
    }

    c.innerHTML = str;
}


function update_sinks(ch_in, ch_out) {
    var o = document.getElementById('outputs');
    var r = Math.floor((ch_out%64)/8);
    var c = ch_out%8;
    var str = "";
    var bg, fg, opa;

    opa = (parent.frames[0].config_lock[ch_in])?0.5:1.0;
    if (ch_in>=0) {
				bg = get_sync_color(sync_check(ch_in, ch_out, true));
				if (labels) {
						str = parent.frames[0].labels_audio_in[ch_in];
						if (bg=='#eee')
								bg = '#'+parent.frames[0].labels_audio_in_bg[ch_in];
						fg = '#'+parent.frames[0].labels_audio_in_fg[ch_in];
				} else {
						str = (Math.floor(ch_in/64)+1)+' / '+((ch_in)%64+1);
						fg = '#333';
				}
    } else {
				str = '---';
				bg = '#eee';
				fg = '#333';
    }
    
    o.children[0].children[r].children[c+1].innerHTML = "<div class='label' style='background-color: "+bg+"; color: "+fg+"; opacity: "+opa+";'>"+str+"</div>";
}


function update_sources(ch_in) {
    var o = document.getElementById('inputs');
    var r = Math.floor((ch_in%64)/8);
    var c = ch_in%8;
    var label;
    var str = "";
    var bg, fg, opa;
    
    for (var i=0; i<1024; i++) {
				if (parent.frames[0].config_xpa[i]==ch_in) {
						opa = (parent.frames[0].config_lock[i])?0.5:1.0;
						bg = get_sync_color(sync_check(ch_in, i, false));
						
						if (labels) {
								label = parent.frames[0].labels_audio_out[i];	
								if (bg=='#eee')
										bg = '#'+parent.frames[0].labels_audio_out_bg[i];
								fg = '#'+parent.frames[0].labels_audio_out_fg[i];
						} else {
								label = (Math.floor(i/64)+1)+' / '+((i)%64+1);
								fg = '#333';
						}
						str += "<div class='label' style='background-color: "+bg+"; color: "+fg+"; opacity: "+opa+";'>"+label+"</div>";
				}
    }
    if (str=="") {
				str = "---";
    }

    o.children[0].children[r].children[c+1].innerHTML = str;
}


function remove_xp(m, ch_in, ch_out) {
    if (m==2)
				return;
    if ((ch_in>=port*64) && (ch_in<(port+1)*64)) {
				update_sources(ch_in);
    }
}


var labels=false;

function table_update_sync() {
    for (var i=(port)*64; i<(port+1)*64; i++) {
				update_sources(i);
				update_sinks(parent.frames[0].config_xpa[i], i);
    }
}


function table_update_geometry(t, ch, mul) {
    t.children[0].children[2].style.display = (mul<2)?'table-row':'none';
    t.children[0].children[3].style.display = (mul<2)?'table-row':'none';
    t.children[0].children[4].style.display = (mul<1)?'table-row':'none';
    t.children[0].children[5].style.display = (mul<1)?'table-row':'none';
    t.children[0].children[6].style.display = (mul<1)?'table-row':'none';
    t.children[0].children[7].style.display = ((mul==0) && (ch==64))?'table-row':'none';
    
    t.children[0].children[3].children[5].style.display = ((mul==1) && (ch==56))?'none':'table-cell';
    t.children[0].children[3].children[6].style.display = ((mul==1) && (ch==56))?'none':'table-cell';
    t.children[0].children[3].children[7].style.display = ((mul==1) && (ch==56))?'none':'table-cell';
    t.children[0].children[3].children[8].style.display = ((mul==1) && (ch==56))?'none':'table-cell';				  
    
    t.children[0].children[1].children[7].style.display = ((mul==2) && (ch==56))?'none':'table-cell';
    t.children[0].children[1].children[8].style.display = ((mul==2) && (ch==56))?'none':'table-cell';
}


function esc_xp(m, ch_in, ch_out) {
    if (m==2) 
				return;
    if ((ch_out>=port*64) && (ch_out<(port+1)*64)) {
				update_sinks(ch_in, ch_out);
    }
    
    if ((ch_in>=port*64) && (ch_in<(port+1)*64)) {
				update_sources(ch_in);
    }
}

function esc_al(ch, val) {
    if ((ch>=port*64) && (ch<(port+1)*64)) {
				update_sinks(parent.frames[0].config_xpa[ch], ch);
    }
    for (var i=(port)*64; i<(port+1)*64; i++) {
				update_sources(i);
    }
}

function esc_auto_57ch(val) {
    update_mode_switches();
}

function ess_freq(p, freq) {
    if (p==port) {
				if (parent.frames[0].status_port_sync[port]>0) {
						document.getElementById("input_freq").innerHTML = freq+" Hz";
				} else {
						document.getElementById("input_freq").innerHTML = "-";
				}
    }
}


function esc_poly_source(p, source) {
    if (p==port) {
				set_selectactive(document.getElementById("ref_clock"), source);		
    }
    update_redundancy_details();
    
    update_clocks();
    
    table_update_sync();
}


function esc_master_enable_clock(enable) {
    update_clocks();
    update_tables();
}


function esc_master_enable_fs(enable) {
    update_clocks();
    update_tables();
}


function sfp_status(modules, levels) {
    var str = "";
     if (modules[port]>0) {
				if (modules[port]&0x40) {
						str += ((levels[port]-400)/10)+"dB";
						if (modules[port]&0x02) {
								str += " <b style='color: red;'>LOW ALARM</b>";
						} else if (modules[port]&0x01) {
								str += " <b style='color: red;'>HIGH ALARM</b>";
						} else if (modules[port]&0x08) {
								str += " <b style='color: yellow;'>LOW WARNING</b>";
						} else if (modules[port]&0x04) {
								str += " <b style='color: yellow;'>HIGH WARNING</b>";
						}
				} else {
						str = "[module has no DMM capabilities]";
						document.getElementById("sfp_graph").style.opacity = 0.5;
				}
				document.getElementById("sfp_details").disabled = false;
				
				var g = document.getElementById("sfp_graph_bar");
				if (g) {
						document.getElementById("sfp_graph").style.opacity = 1;
						g.style.left = (levels[port]/2-5)+"px";
				}
				
    } else {
				str = "-";
				document.getElementById("sfp_details").disabled = true;
				document.getElementById("sfp_graph").style.opacity = 0.5;
    }
    
    document.getElementById("sfp_level").innerHTML = str;

    oldmodules = modules;
    oldlevels = levels;
}


function cb_sfp_thresholds(str) {
    if (str=="")
				return;
    
    var data = JSON.parse(str);

    var o = document.getElementById("sfp_graph_low_alarm");
    var a = (parseFloat(data["rx_power_low_alarm"])+40)*5;
    o.style.width = a+"px";
    
    o = document.getElementById("sfp_graph_low_warning");
    o.style.left = a+"px";
    var w = (parseFloat(data["rx_power_low_warning"])+40)*5;
    o.style.width = (w-a)+"px";
    
    o = document.getElementById("sfp_graph_high_warning");
    w = (parseFloat(data["rx_power_high_warning"])+40)*5;
    o.style.left = w+"px";
    
    o = document.getElementById("sfp_graph_high_alarm");
    a = (parseFloat(data["rx_power_high_alarm"])+40)*5;
    o.style.left = a+"px";
}


function update_title() {
    if (labels) {
				document.getElementById("title").innerHTML = "Port "+(port+1)+": "+parent.frames[0].labels_input_ports[port]+" / "+parent.frames[0].labels_output_ports[port];
    } else {
				document.getElementById("title").innerHTML = "Port "+(port+1);
    }
}


function update_tables() {
    table_update_geometry(document.getElementById('inputs'), parent.frames[0].status_port_mode[port], parent.frames[0].get_effective_mul(port));
    table_update_geometry(document.getElementById('outputs'), parent.frames[0].config_port_mode[port], parent.frames[0].get_effective_mul(port));
    table_update_sync();
}


function update_selects() {
    var bla = parent.frames[0].config_port_input_redundancy;
    var r_i = document.getElementById("redundancy_in");
    var r_o = document.getElementById("redundancy_out");
    var f = document.getElementById("follow_port");
    set_selectactive(r_i, parent.frames[0].config_port_input_redundancy[port]);
    if (r_o)
				set_selectactive(r_o, parent.frames[0].config_port_output_redundancy[port]);
    set_selectactive(f, parent.frames[0].config_port_follow[port]);
    for (var i=0; i<16; i++) {
				if (i==port)
						continue;

				r_i.children[i+1].disabled = (parent.frames[0].config_port_input_redundancy[i] != i);
				if (r_o)
						r_o.children[i+1].disabled = (parent.frames[0].config_port_output_redundancy[i] != i);
				f.children[i+2].disabled = (parent.frames[0].config_port_follow[i] != i);
				if (labels) {
						r_i.children[i+1].innerHTML = parent.frames[0].labels_input_ports[i];
						if (r_o)
								r_o.children[i+1].innerHTML = parent.frames[0].labels_input_ports[i];
						f.children[i+2].innerHTML = parent.frames[0].labels_input_ports[i];
				} else {
						r_i.children[i+1].innerHTML = 'Port '+(i+1);
						if (r_o)
								r_o.children[i+1].innerHTML = 'Port '+(i+1);
						f.children[i+2].innerHTML = 'Port '+(i+1);
				}
    }
}


function permissions_updated() {
    document.getElementById("label_clock").className = (parent.frames[0].permissions.clocks[port]==0)?"disabled":"";
    document.getElementById("ref_clock").disabled = (parent.frames[0].permissions.clocks[port]==0);
    document.getElementById("label_mul").className = (parent.frames[0].permissions.port_mul[port]==0)?"disabled":"";
    document.getElementById("values_mul").className = (parent.frames[0].permissions.port_mul[port]==0)?"disabled":"";
    document.getElementById("port_mul_0").disabled = (parent.frames[0].permissions.port_mul[port]==0);
    document.getElementById("port_mul_1").disabled = (parent.frames[0].permissions.port_mul[port]==0);
    document.getElementById("port_mul_2").disabled = (parent.frames[0].permissions.port_mul[port]==0);
    document.getElementById("label_follow").className = (parent.frames[0].permissions.port_follow[port]==0)?"disabled":"";
    document.getElementById("follow_port").disabled = (parent.frames[0].permissions.port_follow[port]==0);
    document.getElementById("label_redundancy").className = (parent.frames[0].permissions.redundancy_in[port]==0)?"disabled":"";
    document.getElementById("redundancy_in").disabled = (parent.frames[0].permissions.redundancy_in[port]==0);
    var m = document.getElementById("label_mirror");
    if (m)
				m.className = (parent.frames[0].permissions.redundancy_out[port]==0)?"disabled":"";
    var r_o = document.getElementById("redundancy_out");
    if (r_o)
				r_o.disabled = (parent.frames[0].permissions.redundancy_out[port]==0);
    update_mode_switches();
    document.getElementById("label_frame").className = (parent.frames[0].permissions.port_frame[port]==0)?"disabled":"";
    document.getElementById("values_frame").className = (parent.frames[0].permissions.port_frame[port]==0)?"disabled":"";
    document.getElementById("frame_48").disabled = (parent.frames[0].permissions.port_frame[port]==0);
    document.getElementById("frame_96").disabled = (parent.frames[0].permissions.port_frame[port]==0);
    update_port_follow();
}


function show_labels(v) {
    if (typeof(v)=='string') {
				v = parseInt(v, 10);
				var bla=typeof(v);
				if ((typeof(v)!='number') || isNaN(v))
						v = 0;
				set_cycle_button(document.getElementById("show_labels"), v);
    } else if (labels != v) {
				post_call('/wwwdata?show_labels', v, false);
    }

    labels = v;

    update_title();

    for (var i=(port)*64; i<(port+1)*64; i++) {
				update_sources(i);
				update_sinks(parent.frames[0].config_xpa[i], i);
    }
    
    update_selects();
}


function esc_wck_mul(p, mul) {
    update_tables();
    if (p==port) {
				document.getElementById("port_mul_"+mul).checked = true;
    } 
    update_redundancy_details();
    update_clocks();
}


function ess_port_mode(p, mode) {
    if (p==port) {
				if (parent.frames[0].status_port_sync[port]>0) {
						document.getElementById("input_mode").innerHTML = mode+"ch";
				} else {
						document.getElementById("input_mode").innerHTML = "-";
				}
				table_update_geometry(document.getElementById('inputs'), mode, parent.frames[0].get_effective_mul(port));
    }
    update_redundancy_details();
    update_mode_switches();
}


function ess_port_frame(p, frame) {
    if (p==port) {
				if (parent.frames[0].status_port_sync[port]>0) {
						document.getElementById("input_frame").innerHTML = frame+"k";
				} else {
						document.getElementById("input_frame").innerHTML = "-";
				}
    }
    update_redundancy_details();
}


function esc_port_input_redundancy(p, redundancy) {
    update_selects();

    if (p==port) {
				update_redundancy_details();
    }
}


function update_mode_switches() {
    var alloff = (parent.frames[0].permissions.port_mode[port]==0) ||
				((parent.frames[0].status_port_mode[port]==57) && (parent.frames[0].config_auto_57ch==1)) ||
				(parent.frames[0].config_port_input_redundancy[port]!=port);
    
    document.getElementById("label_mode").className = alloff?"disabled":"";
    document.getElementById("values_mode").className = alloff?"disabled":"";
    document.getElementById("mode_56").disabled = alloff;
    document.getElementById("mode_57").disabled = alloff || 
				(parent.frames[0].config_auto_57ch==1);
    document.getElementById("label_57").className = (alloff || 
																										 (parent.frames[0].config_auto_57ch==1))?"disabled":"";
    document.getElementById("mode_64").disabled = alloff;

    document.getElementById("auto57_warning").style.display =
				(parent.frames[0].config_auto_57ch==1)?"block":"none";
}


function esc_port_output_redundancy(p, redundancy) {
    if (p==port) {
				if (perm_mode[port]==1) {
						update_mode_switches();
				}
				if (perm_frame[port]==1) {
						document.getElementById('frame_48').disabled = (redundancy!=port);
						document.getElementById('frame_96').disabled = (redundancy!=port);
				}
    }
}


function update_port_follow() {

    var i;
    for (i=0; i<16; i++) {
				if (i==port)
						continue;

				if (parent.frames[0].config_port_follow[i]==port)
						break;
    }

    if ((parent.frames[0].permissions) && (parent.frames[0].permissions.port_follow[port]==1))
				document.getElementById('follow_port').disabled = (i!=16);

    update_clocks();
}


function esc_port_follow(p, follow) {
    var f = document.getElementById("follow_port");

    if (p==port) {
				set_selectactive(f, follow);
				if (p==follow) {
						f.children[p+2].disabled = true;
				}

				update_port_follow();

    } else {
				f.children[p+2].disabled = (p!=follow);
    }

    if (follow==port)
				update_port_follow();

    update_clocks();
}


function esc_port_mode(p, mode) {
    if (p==port) {
				document.getElementById("mode_"+mode).checked = true;
				table_update_geometry(document.getElementById('outputs'), mode, parent.frames[0].get_effective_mul(port));
    }
}

function esc_port_frame(p, frame) {
    if (p==port) {
				document.getElementById("frame_"+frame).checked = true;
    }
}

function input_color(cc) {
    var o = document.getElementById('inputs');
    var r, c;
    
    for (r=0; r<8; r++) {
				for (c=1; c<9; c++) {
						o.children[0].children[r].children[c].style.backgroundColor = cc;
				}    
    }
}


function ess_sync(p, sync, t) {
    sync_update_rate = 1;
    ajax_call('/data/sync_history?'+port, sync_log_data);
    update_redundancy_details();
    
    update_tables();
    
    if (p!=port)
				return;
    
    if (sync==3)
				document.getElementById("sync_status").style.color = 'black';
    
    update_clocks();
    /*
      sync_table.push([new Date().getTime()/1000, sync_last, sync]);
      sync_log_draw();
      sync_table_update();
      sync_last = sync;
    */
    
}


function update_clocks() {
    document.getElementById('clock_ref').innerHTML = parent.frames[0].wck_sources[parent.frames[0].get_effective_source(port)];
    document.getElementById('speed_mode').innerHTML = parent.frames[0].fs_strings[parent.frames[0].get_effective_mul(port)];
    
    document.getElementById('info_master_clock').style.display = (parent.frames[0].config_master_enable_clock)?'block':'none';
    document.getElementById('info_master_fs').style.display = (parent.frames[0].config_master_enable_fs)?'block':'none';
    
    if (parent.frames[0].permissions) {
				if (parent.frames[0].permissions.clocks[port]==1)
						document.getElementById('ref_clock').disabled = parent.frames[0].config_master_enable_clock || (port!=parent.frames[0].config_port_follow[port]);
				if (parent.frames[0].permissions.port_mul[port]==1) {
						document.getElementById('port_mul_0').disabled = parent.frames[0].config_master_enable_fs || (port!=parent.frames[0].config_port_follow[port]);
						document.getElementById('port_mul_1').disabled = parent.frames[0].config_master_enable_fs || (port!=parent.frames[0].config_port_follow[port]);
						document.getElementById('port_mul_2').disabled = parent.frames[0].config_master_enable_fs || (port!=parent.frames[0].config_port_follow[port]);
				}
    }
}


//
//
//


function history_set_scale(scale) {
    parent.frames[0].history_scale = scale;
    var b = document.getElementById('btn_history_scale');

    if (scale==0) {
				b.innerHTML = "All";
    } else if (scale==60) {
				b.innerHTML = "1m";
    } else if (scale==300) {
				b.innerHTML = "5m";
    } else if (scale==900) {
				b.innerHTML = "15m";
    } else {
				b.innerHTML = "??";
    }

    sync_log_draw();
}


//
// buttons
//


function change_scale() {
    if (parent.frames[0].history_scale==0) {
				history_set_scale(60);
    } else if (parent.frames[0].history_scale==60) {
				history_set_scale(300);
    } else if (parent.frames[0].history_scale==300) {
				history_set_scale(900);
    } else {
				history_set_scale(0);
    }
}


function history_cleared() {
    sync_log_init(port);
}


function history_clear(port) {
    ajax_call('/data/sync_clear?'+port, history_cleared);
    
}


//
// initialization
//
var sync_update = 0;

function update_sync() {
    document.getElementById("sync_status").innerHTML = parent.frames[0].status_sync_str(port);
    window.setTimeout('update_sync()', 1000);
    
    sync_update++;

    if (sync_update>=sync_update_rate) {
				sync_log_draw();
				sync_update = 0;
    }
}


function cb_keydown(e) {
    switch(e.keyCode) {
    case 37: // CL
				if (port>0) init_port(--port); 
				e.preventDefault();
				break;
				
    case 39: // CR
				if (port<15) init_port(++port);
				e.preventDefault();
				break;

    case 68:
				push_cycle_button(document.getElementById("show_labels"));    
				e.preventDefault();
				break;
    }
}


function init_port(port) {
		if (port<0)
				port = 0;
		if (port>15)
				port = 15;
		
    parent.frames[0].set_pagetitle("Port "+(port+1));

		parent.frames[0].last_port = port;

		sync_log_init(port);

    update_sync();

    if (3 != parent.frames[0].status_port_sync[port]) {
				document.getElementById("sync_status").style.color = 'red';
    } else {
				document.getElementById("sync_status").style.color = 'black';
    }
    
    ess_freq(port, parent.frames[0].status_port_freq[port]);
    esc_poly_source(port, parent.frames[0].config_poly_source[port]);
    esc_wck_mul(port, parent.frames[0].config_wck_mul[port]);
    ess_port_mode(port, parent.frames[0].status_port_mode[port]);
    ess_port_frame(port, parent.frames[0].status_port_frame[port]);
    
    document.getElementById("mode_"+parent.frames[0].config_port_mode[port]).checked = true;
    document.getElementById("frame_"+parent.frames[0].config_port_frame[port]).checked = true;
    
    var r_i = document.getElementById("redundancy_in");
    var r_o = document.getElementById("redundancy_out");
    var f = document.getElementById("follow_port");
    
    for (var i=0; i<17; i++) {
				r_i.children[i].disabled = (port+1==i);
				if (r_o)
						r_o.children[i].disabled = (port+1==i);
				f.children[i+2].disabled = (port==i);
    }
    r_i.children[0].value = port;
    if (r_o)
				r_o.children[0].value = port;
    f.children[0].value = port;
    
    update_selects();
    update_title();
    update_port_follow();
    update_redundancy_details();
    
    btn_prev = document.getElementById("previous_port");
    btn_next = document.getElementById("next_port");

    if (port==0) {
				btn_prev.style.opacity = 0.5;
				btn_prev.style.cursor = 'default';
    } else {
				btn_prev.style.opacity = 1;
				btn_prev.style.cursor = 'pointer';
    }
    if (port==15) {
				btn_next.style.opacity = 0.5;
				btn_next.style.cursor = 'default';
    } else {
				btn_next.style.opacity = 1;
				btn_next.style.cursor = 'pointer';
    }
    
    for (var i=(port)*64; i<(port+1)*64; i++) {
				update_sources(i);
				update_sinks(parent.frames[0].config_xpa[i], i);
    }
    
    table_update_geometry(document.getElementById('inputs'), parent.frames[0].status_port_mode[port], parent.frames[0].get_effective_mul(port));
    table_update_geometry(document.getElementById('outputs'), parent.frames[0].config_port_mode[port], parent.frames[0].get_effective_mul(port));
    
    esc_port_follow(port, parent.frames[0].config_port_follow[port]);
    update_port_follow();

    esc_auto_57ch(parent.frames[0].config_auto_57ch);

    history_set_scale(parent.frames[0].history_scale);
    
    permissions_updated();

    if (parseInt(parent.frames[0].status_io_ports[Math.floor(port/8)][port%8], 10)==7) {
				document.getElementById("sfp_panel").style.display = "block";
				if (oldmodules)
						sfp_status(oldmodules, oldlevels);

				ajax_call("/data/sfpl?"+port, cb_sfp_thresholds);
		} else {
				document.getElementById("sfp_panel").style.display = "none";
		}

}


function port_onload() {
		ajax_call('/wwwdata?show_labels', show_labels);

		if (window.location.search.length>0) {
				port = window.location.search.substr(1)-1;
		} else if (parent.frames[0].last_port) {
				port = parent.frames[0].last_port;
		} else {
				port = 0;
		}
		init_port(port);
		
    init_cycle_button(document.getElementById("show_labels"), ['System Labels', 'User Labels'], show_labels);  
    window.onkeydown = cb_keydown;
}
